"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    LoopKind: ()=>LoopKind,
    SchedulerEvent: ()=>SchedulerEvent,
    SchedulerTarget: ()=>SchedulerTarget,
    SchedulerEventGroup: ()=>SchedulerEventGroup,
    SchedulerEventGroupNode: ()=>SchedulerEventGroupNode
});
const _everydayUtils = require("everyday-utils");
const _immutableMapSet = require("immutable-map-set");
const _eventUtil = require("./event-util");
const _messageQueue = require("./message-queue");
const LoopKind = {
    Once: 0,
    Loop: 1,
    Live: 2
};
let SchedulerEvent = class SchedulerEvent {
    id = (0, _everydayUtils.cheapRandomId)();
    midiEvent = {
        data: new Uint8Array(3),
        receivedTime: 0
    };
    constructor(data = {}){
        this.id = data.id ?? this.id;
        if (data.midiEvent) {
            this.midiEvent.data = data.midiEvent.data;
            this.midiEvent.receivedTime = data.midiEvent.receivedTime;
        }
    }
    toJSON() {
        return {
            id: this.id,
            midiEvent: {
                receivedTime: this.midiEvent.receivedTime ?? 0,
                data: this.midiEvent.data
            }
        };
    }
};
let SchedulerTarget = class SchedulerTarget {
    id = (0, _everydayUtils.cheapRandomId)();
    midiQueue = new _messageQueue.MessageQueue().clear();
    constructor(data = {}){
        Object.assign(this, data);
    }
};
let SchedulerEventGroup = class SchedulerEventGroup {
    id = (0, _everydayUtils.cheapRandomId)();
    targets = new _immutableMapSet.ImmSet();
    scheduler;
    loopPoints = new Float64Array(new SharedArrayBuffer(3 // [loopActive, loopStart, loopEnd]
     * Float64Array.BYTES_PER_ELEMENT));
    constructor(eventGroup = {}){
        Object.assign(this, eventGroup);
    }
    toJSON() {
        return {
            id: this.id,
            targets: this.targets,
            loopPoints: this.loopPoints
        };
    }
    get loop() {
        return this.loopPoints[0];
    }
    set loop(value) {
        this.loopPoints[0] = value;
    }
    get loopStart() {
        return this.loopPoints[1];
    }
    set loopStart(seconds) {
        this.loopPoints[1] = seconds;
    }
    get loopEnd() {
        return this.loopPoints[2];
    }
    set loopEnd(seconds) {
        this.loopPoints[2] = seconds;
    }
    setMidiEvents(turnEvents, turn = 0, clear) {
        const turns = turnEvents.map(()=>new Set());
        for (const [i, midiEvents] of turnEvents.entries()){
            const events = turns[i];
            for (const midiEvent of midiEvents){
                const event = new SchedulerEvent({
                    midiEvent
                });
                events.add(event);
            }
        }
        try {
            this.scheduler?.worklet.receiveEvents(this.id, turn, turns, clear);
        } catch (error) {
            console.warn(error);
        }
        return turnEvents;
    }
    setNotes(turnNotes, turn = 0, clear) {
        const midiEvents = turnNotes.map((note)=>(0, _eventUtil.getMidiEventsForNotes)(note));
        return this.setMidiEvents(midiEvents, turn, clear);
    }
};
let SchedulerEventGroupNode = class SchedulerEventGroupNode extends EventTarget {
    eventGroup;
    targetNodes;
    constructor(schedulerNode){
        super();
        this.schedulerNode = schedulerNode;
        this.eventGroup = new SchedulerEventGroup();
        this.targetNodes = new Set();
        this.schedulerNode.addEventGroup(this.eventGroup);
    }
    destroy() {
        this.schedulerNode.removeEventGroup(this.eventGroup);
    }
    suspend(targetNode) {
        this.schedulerNode?.worklet.suspendTarget(targetNode.id);
    }
    resume(targetNode) {
        this.schedulerNode?.worklet.resumeTarget(targetNode.id);
    }
    clear() {
        this.schedulerNode?.worklet.clearEventGroup(this.eventGroup.id);
    }
    connect(targetNode) {
        this.schedulerNode.connect(targetNode);
        this.targetNodes.add(targetNode);
        this.eventGroup.targets = this.eventGroup.targets.add(targetNode.schedulerTarget);
        this.dispatchEvent(new CustomEvent('connectchange'));
        return targetNode;
    }
    disconnect(targetNode) {
        this.schedulerNode.disconnect(targetNode);
        this.targetNodes.delete(targetNode);
        this.eventGroup.targets = this.eventGroup.targets.delete(targetNode.schedulerTarget);
        this.dispatchEvent(new CustomEvent('connectchange'));
    }
    schedulerNode;
};

//# sourceMappingURL=scheduler-event.js.map