"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "SchedulerTargetNode", {
    enumerable: true,
    get: ()=>SchedulerTargetNode
});
const _aliceBob = require("alice-bob");
const _everydayUtils = require("everyday-utils");
const _schedulerEvent = require("./scheduler-event");
const xfer = (args)=>{
    const xfers = [];
    for (const arg of args){
        if (arg instanceof MessagePort) {
            xfers.push(arg);
        }
    }
    return xfers;
};
let SchedulerTargetNode = class SchedulerTargetNode extends AudioWorkletNode {
    id = (0, _everydayUtils.cheapRandomId)();
    schedulerTarget = new _schedulerEvent.SchedulerTarget();
    worklet;
    constructor(context, name, options = {}){
        // add a phony tail input, used for connecting the
        // scheduler to execute right before our worklet
        options.numberOfInputs = (options.numberOfInputs ?? 0) + 1;
        super(context, name, options);
        const [node, worklet] = new _aliceBob.Alice((data)=>void this.port.postMessage(data, xfer(data.args))).agents({
            debug: false
        });
        this.port.onmessage = ({ data  })=>node.receive(data);
        // TODO: handle onprocessorerror
        this.worklet = worklet;
    }
    async init() {
        await this.worklet.init(this.schedulerTarget.midiQueue.buffer);
    }
    processMidiEvent(midiEvent) {
        if (midiEvent.receivedTime == null) {
            throw new Error('Midi event missing receivedTime');
        }
        this.schedulerTarget.midiQueue.push(midiEvent.receivedTime >= 0 ? midiEvent.receivedTime : this.context.currentTime, ...midiEvent.data);
    }
};

//# sourceMappingURL=scheduler-target-node.js.map