import { Alice } from 'alice-bob';
import { queue } from 'event-toolkit';
import { SyncedSet } from 'synced-set';
import { core } from './scheduler-core';
export class SchedulerNode extends EventTarget {
    static registeredCtx = new WeakSet();
    static async register(context) {
        if (this.registeredCtx.has(context)) return;
        // @ts-ignore
        await context.audioWorklet.addModule(new URL('./scheduler-processor.js', import.meta.url).href);
        this.registeredCtx.add(context);
    }
    static async create(context) {
        await this.register(context);
        return new this(context);
    }
    node;
    worklet;
    targetNodes;
    eventGroups;
    addEventGroup(eventGroup) {
        eventGroup.scheduler = this;
        this.eventGroups.add(eventGroup);
        return eventGroup;
    }
    removeEventGroup(eventGroup) {
        this.eventGroups.delete(eventGroup);
    }
    requestNextEvents(eventGroupId, turn = 0, total = 4) {
        const eventGroup = [
            ...this.eventGroups
        ].find((eventGroup)=>eventGroup.id === eventGroupId);
        if (!eventGroup) {
            console.warn(`Event group with id "${eventGroupId}" not found`);
            return;
        }
        eventGroup.onRequestNotes?.(turn, total);
    }
    constructor(context){
        super();
        this.context = context;
        this.targetNodes = new Set();
        this.eventGroups = new SyncedSet({
            send: queue.task((payload, cb)=>{
                //!? 'sending'
                this.node.port.postMessage({
                    eventGroups: core.serialize(payload)
                });
                cb();
            }),
            pick: [],
            reducer: (eventGroup)=>({
                    targets: eventGroup.targets
                }),
            equal: (prev, next)=>prev.targets === next.targets
        });
        this.node = new AudioWorkletNode(this.context, 'scheduler');
        const [scheduler, worklet] = new Alice((data)=>void this.node.port.postMessage({
                rpc: data
            }), this).agents({
            debug: false
        });
        this.node.port.onmessage = ({ data  })=>{
            if (data.rpc) scheduler.receive(data.rpc);
        };
        // TODO: handle onprocessorerror
        this.worklet = worklet;
    }
    start(playbackStartTime, offsetStartTime) {
        return this.worklet.start(playbackStartTime, offsetStartTime);
    }
    stop() {
        return this.worklet.stop();
    }
    setBpm(bpm) {
        return this.worklet.setBpm(bpm);
    }
    setClockBuffer(clockBuffer) {
        return this.worklet.setClockBuffer(clockBuffer);
    }
    connect(targetNode) {
        if (!this.targetNodes.has(targetNode)) {
            this.node.connect(targetNode, 0, targetNode.numberOfInputs - 1);
        }
        return targetNode;
    }
    disconnect(targetNode) {
        if (this.targetNodes.has(targetNode)) {
            this.node.disconnect(targetNode, 0, targetNode.numberOfInputs - 1);
        }
    }
    context;
}

//# sourceMappingURL=scheduler-node.js.map